#!/bin/bash

#set -x
smlog()
{
   echo $( date +"%Y-%m-%d %T:" ) ssb-3500: Info: "$1" >&2
}

smerr()
{
   echo $( date +"%Y-%m-%d %T:" ) ssb-1501: ERROR: "$1" >&2
}

usage()
{
   cat << EOF

   SEP sesam SAP HANA 2.0 Backup and Housekeeping Script

          usage:

          sbc_hana.sh <ACTION> <OPTIONS> <HANA>

          Available actions:
             -b backup (default)
             -A purge: HANA Backup Catalog Housekeeping

          OPTIONS:
             for backup
                -l Level, full/diff/inc
                -U User or Userkey
                -P Password for User
                -o purge: Start HANA Backup Catalog Housekeeping after FULL backup
                -C Sesam server: Sesam server to contact for housekeeping
                -J Task event: Task event related to backup for housekeeping
                -j Task: Task related to backup for housekeeping


             for housekeeping
                -U User or Userkey
                -P Password for User
                -s HANA Backup ID (BID)
                -C Sesam server: Sesam server to contact for housekeeping
                -J Task event: Task event related for housekeeping
                -j Task: Task related for housekeeping

          HANA
          <SID>/<INSTANCE>/<DATABASE>

          sbc_hana.sh -b -l <full|diff|incr> -U <user> -P <password> <SID>/<Instance>/<DB>


          Example: FULL backup of database 'PRD' with SID 'HDB' and instance 99
          User with access to the SYSTEMDB is 'bckadm' and password 'secret'

            sbc_hana.sh -b -l full -U bckadm -P secret HDB/99/PRD


          Example: Incremental backup of database 'DEV' with SID 'HX4' and Instance 01
          Userkey for access the SYSTEMDB is 'bckuserkey'

            sbc_hana.sh -b -l incr -U bckuserkey H4X/01/DEV


          Example: FULL backup with Housekeeping of database 'PRD' with SID 'HDB' and instance 99
          User with access to the SYSTEMDB is 'bckadm' and password 'secret'
          Sesam server is backup-srv and Task is HANA-PRD (get oldest FULL from Sesam server)

            sbc_hana.sh -b -l full -o purge -C backup-srv -j HANA-PRD -U bckadm -P secret HDB/99/PRD


          Example: Housekeeping for database 'DEV' with SID 'HX4' and instance 01
          Userkey for access to SYSTEMDB is 'bckuserkey'
          Sesam server is backup-srv and Task is HANA-DEV (get oldest FULL from Sesam server)

            sbc_hana.sh -A purge -C backup-srv -j HANA-DEV -U bckuserkey H4X/01/DEV

          Example: FULL backup with Housekeeping of database 'PRD' with SID 'HDB' and instance 99
          User with access to the SYSTEMDB is 'bckadm' and password 'secret'
          (get oldest FULL from file, written by last FULL backup process)

            sbc_hana.sh -b -l full -o purge -U bckadm -P secret HDB/99/PRD


          Example: Housekeeping for database 'DEV' with SID 'HX4' and instance 01
          Userkey for access to SYSTEMDB is 'bckuserkey'
          (get oldest FULL from file, written by last FULL backup process)

            sbc_hana.sh -A purge -U bckuserkey H4X/01/DEV


          Example: Housekeeping for database 'DEV' with SID 'HX4' and instance 01, Backup ID 1567001886606
          Userkey for access to SYSTEMDB is 'bckuserkey'
          (remove all backups older then BID 1567001886606 from HANA Backup Catalog)

            sbc_hana.sh -A purge -U bckuserkey -s 1567001886606 H4X/01/DEV
EOF

   exit 1
}

source /etc/sesam2000.ini >/dev/null 2>&1

# getting all option.

while getopts bl:A:a:C:c:O:o:i:I:J:j:t:f:d:S:s:v:P:T:X:x:n:L:C:F:U:N opt
do
      case $opt in
         A) ACTION=$OPTARG;;
         a) x=$OPTARG;;
         b) ACTION='backup';;
         C) SESAM_SERVER=$OPTARG;;
         l) LEVEL=$OPTARG;;
         I) INST=$OPTARG;;
         U) BUSER=$OPTARG;;
         P) BPASSWORD=$OPTARG;;
         S) x=$OPTARG;;
         d) x=$OPTARG;;
         v) x=$OPTARG;;
         s) SSID=$OPTARG;;
         t) x=$OPTARG;;
         i) x=$OPTARG;;
         j) SESAM_JOB=$OPTARG;;
         J) SESAM_TASKEVENT=$OPTARG;;
         L) x=$OPTARG;;
         N) NOEXEC=1;;
         O) x=$OPTARG;;
         o) OPTIONS=$OPTARG;;
         F) x=$OPTARG;;
         ?) usage
          exit 1;;

      esac
done

shift `expr $OPTIND - 1`

SRC=$*

if [[ $SRC != ???/[0-9][0-9]/?* ]]
then
        smerr "Wrong or missing SAP HANA Parameter $SRC"
        echo ""
        echo "The command must end with <SID>/<INSTANCE>/<DATABASE>"
        echo "                          <3 signs>/<2 digits>/<any signs>"
        usage
        exit 1
fi



smlog "HANA Backup / Housekeeping source: $SRC"

# add a slash for correct split during read
SRC="/"$SRC

OIFS=$IFS
IFS='/' read x DBSID INST DBNAME <<<${SRC}
IFS=$OIFS

# Evaluate parameter values
if [ -z $DBSID ]
  then
   smerr "System Identifier (SID) for the System is missing."
   echo ""
   echo "System Identifier (SID) for the System is missing ! "
   usage
   exit 1
fi
   # convert SID in upper case, path part is always uppercase
   DBSID=$(echo $DBSID | tr '[:lower:]' '[:upper:]')
   smlog "Backing up database from SID $DBSID"

if [ -z $BUSER ]
then
      smerr "Backup User or Userkey is missing."
      echo ""
      echo "Backup User or Userkey is missing !"
      usage
      exit 1
fi

if ! [[ $INST =~ ^[0-9]+$ ]]
then
   smerr "HANA Instance number wrong or not specified."
   echo ""
   echo "HANA Instance number wrong or not specified. "
   usage
   exit 1
fi

smlog "Backing up database from SID $DBSID from HANA Instance $INST"

# creating path for executable hdbsql binary
HDBSQL=/usr/sap/$DBSID/HDB$INST/exe/hdbsql

# convert DB Name to uppercase
DBNAME=$(echo $DBNAME | tr '[:lower:]' '[:upper:]')

# insert Instance and DB Name information into log after housekeeping section
smlog "Backing up database from SID $DBSID from HANA Instance $INST"
smlog "Backing up database $DBNAME from SID $DBSID from HANA Instance $INST"
# convert in lowercase
LEVEL=$(echo $LEVEL | tr '[:upper:]' '[:lower:]')


if [ "$ACTION" != "purge" ];then
case $LEVEL in
   full) FDI_LEVEL="";;
   diff) FDI_LEVEL="DIFFERENTIAL";;
   incr) FDI_LEVEL="INCREMENTAL";;
   *) smerr "Backuplevel $LEVEL wrong or unspecified."
      echo ""
      echo "Backuplevel $LEVEL wrong or unspecified. Possible are full or diff or incr."
      usage
   exit 1
   ;;
esac
fi

smlog "Backing up database $DBNAME from SID $DBSID from HANA Instance $INST with level $LEVEL"


# Create a unique backup prefix using the current timestamp
LEVEL=$(echo $LEVEL | tr '[:lower:]' '[:upper:]')
PREFIX=""$LEVEL"_DATA_BACKUP_`date "+%Y-%m-%d_%H-%M-%S"`_`hostname -s`"
if [ ! -z $NOEXEC ];then
  MYEXEC="echo"
fi
# Start backup via hdbsql query
smlog "Starting $LEVEL backup of database $DBNAME from HANA System Identifier $DBSID via hdbsql..."

if [ "$ACTION" = "backup" ]
then
   if [ -z $BPASSWORD ];then
      smlog "Starting $LEVEL backup of database $DBNAME from HANA System Identifier $DBSID via hdbsql with Userkey $BUSER ..."
      smlog "The backup PREFIX is $PREFIX"
      # version with HANA User Store Key User, backupuser is create for SYSTEMDB
      $MYEXEC $HDBSQL -i $INST -d SYSTEMDB -n localhost -U $BUSER "BACKUP DATA $FDI_LEVEL FOR $DBNAME USING BACKINT ('$PREFIX')"
      EXIT_CODE=$?
      set +x
   else
      smlog "Starting $LEVEL backup of database $DBNAME from HANA System Identifier $DBSID via hdbsql with User $BUSER and Password *** ..."
      smlog "The backup PREFIX is $PREFIX"
      set -x
      $MYEXEC $HDBSQL -i $INST -d SYSTEMDB -n localhost -u $BUSER -p $BPASSWORD "BACKUP DATA $FDI_LEVEL FOR $DBNAME USING BACKINT ('$PREFIX')"
      EXIT_CODE=$?
      set +x
   fi
fi
if [ $EXIT_CODE != "0" ]
then
   if [ $EXIT_CODE -eq "191" ]
   then
      smerr "Exit code from hdbsql was 191. This could mean that the tenant $DBSID is not running and has to be started."
      EXIT_CODE=4
   else
      smerr "Exit code $EXIT_CODE from hdbsql"
   fi
exit $EXIT_CODE
fi

## Begin housekeeping section
if [ "$ACTION" = "purge" ] || [[ "$OPTIONS" =~ .*purge.* ]]
then
   f="${SRC//\//@}"
   f="${f:1}"
   PURGE_FILE=$( find /tmp -maxdepth 1 -mtime -2 -name "sesam-purge-$f" )
   if [ -z ${SSID} ]; then
       if [ ! -z "${SESAM_JOB}" ];then
          SSID=$( $SM_BIN_SESAM/sbc_com_interface -q -C $SESAM_SERVER -I sap-hana -j ${SESAM_JOB} -o oldest -v 0 -- | grep "^savesetname="|cut -d = -f 2|grep "^[0-9]*$" )
       elif [ ! -z ${SESAM_TASKEVENT} ]; then
          SSID=$( $SM_BIN_SESAM/sbc_com_interface -q -C $SESAM_SERVER -I sap-hana -J ${SESAM_TASKEVENT} -o oldest -v 0 -- | grep "^savesetname="|cut -d = -f 2|grep "^[0-9]*$" )
       elif [ ! -z ${SESAM_TASKEVENT} ]; then
          SSID=$( $SM_BIN_SESAM/sbc_com_interface -q -C $SESAM_SERVER -I sap-hana -J ${SESAM_TASKEVENT} -o oldest -v 0 -- | grep "^savesetname="|cut -d = -f 2|grep "^[0-9]*$" )
       elif [ ! -z ${PURGE_FILE} ]; then
          smlog "Use backup ID set by recent SAP HANA full backup"
          SSID=$( grep "^savesetname=" ${PURGE_FILE}|cut -d = -f 2|grep "^[0-9]*$" )
       else
          smerr "Could not query Sesam server, because neither backup task nor backup event is set."
          exit 2
       fi
   fi
   if [ -z ${SSID} ]; then
      smlog "No backup found to be purged"
echo "Exit2 $EXIT_CODE"
      exit 0
   fi
   smlog "Oldest backup ID according Sesam DB: ${SSID}"
   if [ -z $BPASSWORD ]
      then
         smlog "Start housekeeping for database $DBNAME from HANA System Identifier $DBSID from HANA Instance $INST via hdbsql with Userkey $BUSER ..."
         smlog "Removing all backups from HANA Backup Catalog older then the BID $SSID"
         # using HANA User Store Key User, backupuser is created for SYSTEMDB
         set -x
         $MYEXEC $HDBSQL -a -j -i $INST -d SYSTEMDB -n localhost -U $BUSER "BACKUP CATALOG DELETE FOR ${DBNAME} ALL BEFORE BACKUP_ID ${SSID}"
         set +x
      else
         smlog "Starting Housekeeping for database $DBNAME from HANA System Identifier $DBSID from HANA Instance $INST via hdbsql with User $BUSER and Password *** ..."
         smlog "Removing all Backups from HANA Backup Catalog older then the BID $SSID"
         # using HANA user and password
         set -x
         $MYEXEC $HDBSQL -a -j -i $INST -d SYSTEMDB -n localhost -u $BUSER -p $BPASSWORD "BACKUP CATALOG DELETE FOR ${DBNAME} ALL BEFORE BACKUP_ID ${SSID}"
         set +x
      fi
fi
## End Housekeeping section
echo "Exit2 $EXIT_CODE"

if [ "$ACTION" = "purge" ]
then
   smlog "SAP Hana housekeeping done."  # No real exit code from Hana
   exit 0
fi

# Collect exit code of hdbsql and pass it as exit code of this backup script
if [ $EXIT_CODE == "0" ]
then
   smlog "Operation successful."
else
   if [ $EXIT_CODE -eq "191" ]
   then
      smerr "Exit code from hdbsql was 191. This could mean that the tenant $DBSID is not running and has to be started."
      EXIT_CODE=4
   else
      smerr "Exit code $EXIT_CODE from hdbsql"
   fi
fi
echo "Exit2 $EXIT_CODE"


exit $EXIT_CODE
