#!/bin/bash

szVersion="\$Id: sbc_oracle_rman.sh,v 1.22 2013/12/04 17:10:30 miraculix Exp $"
###########################################################################
# sbc_oracle_rman.sh
#
# Description: 
# SEP Oracle 9i/10g Backup utility for client based Oracle Backup and Restore 
# RMAN backup type SBT
#
# This utility builds and runs RMAN scripts 
#
# Syntax:
Usage ()
{
 echo "Usage: $0 -b|r <-l level> -T <TargetDBS> -R <Catalog> -S <SesamServer> -j <SesamJob> -m <Mediapool> [-d drive] [-o <option>] [-p <Number of streams>] [<TableSpace>]

 Arguments:
 -b: Backup operation. By default, this is a full, online, single channel backup
 -r: Restore operation. By default, this is a full online, single channel restore.
     HINTS:
       Before restore database must be set in required state, 
       e.g. \"SHUTDOWN; STARTUP MOUNT;\"
       After restore an \"ALTER DATABASE OPEN RESETLOGS;\" may be necessary.

 -l <copy|full|diff|incr|arch>: Backup level copy, full, diff, incr and Archivelog
    Backup level may be combined with archivelog backup, 
    e.g. -l copy_arch

 -o {option}: The following options are supported:

  delete                    Delete archivelogs after backup
  delete_later              Delete archivelogs when successfully backed up 3 times
  offline                   Attention: This option will automatically SHUTDOWN 
                                       database before and STARTUP after backup!

  controlfile               Restore controlfile with rman catalog
  controlfile_DBID={DBID}   Restore controlfile without rman catalog. Needs DBID 
  recover                   Recover after restore
  validate                  Restore validate for database and archive logs
  until={YYYYMMDD-HH:MM:SS} Point-in-time recover with until time

 -T <Target database connect string>:  Default: ORACLE_SID 
 -R <Recovery catalog connect string>: Default: NoCatalog
 -C <SesamServer>: Sesam Server host name
 -S <TapeServer>: Tape Server host name
 -j <SesamJob>: Sesam Job name, under which backup is running
 -v [0|1]: debug on (set -x)
 -m <pool name>: Sesam media pool to use for backup 
 -p <number> defines the number of backup channels
 <TableSpace> defines a single tablespace to backup or restore. 
              If not given, backup/restore whole database
  -u <block_size> - block size in bytes which will be used during backup/restore"
}
#
# Return value: 0 for OK, 1 for error
#
# Messaging:    Forwarded from RMAN
#
###########################################################################

#
# Defaults --- Change with carefulness to your needs!

szMaxSeq="30"
szMaxDays="35"

#
#



my_echo()
{
  echo `date +"%Y-%m-%d %H:%M:%S"`: sob-"$1" >&2
}


exec_rman_cmdfile()
{
  rman cmdfile $1
}


# usage check
if [ $# -lt 5 ]
then
  Usage
  exit 1
fi

# user check
if [ "$USER" = "root" ]
then
  su - oracle -c "$0 $*"
  exit
fi

my_echo "3500: Info: $szVersion"
my_echo "3500: Info: Starting `basename $0` $*"

# Configuration


# RMAN command
szRmanCMD="$ORACLE_HOME/bin/rman"

# Destination directory for DISK backup
#szDiskDestination="$ORACLE_HOME/sob/backup"

szOperation=""

if [[ $1 = "" ]]; 
then
  Usage
  my_echo "1000: Error:    No or wrong parameters. Exiting...."
  exit 1
fi

if [[ "$ORACLE_HOME" = "" ]]
then
  my_echo "1000: Error: ORACLE_HOME environment variable not defined. Exiting."
  exit 1
fi

# logging call of sbc_oracle_rman.log
szRmanLog=$ORACLE_HOME/sob/sbc_oracle_rman.log
echo `date +"%Y-%m-%d %H:%M:%S"`": sob-3500: Info: Starting `basename $0` $*" >>$szRmanLog

# Getting options
sz_opts=":brca:C:S:j:d:f:l:m:o:p:R:t:T:W:v:u:"
while getopts $sz_opts c_opt; do
	case $c_opt in
	b)	szOperation="Backup";;	
	r)	szOperation="Restore";;
	f)	szCmdFile=$OPTARG;; 
	l)	szLevel=$OPTARG;; 
	C)	szSesamServer=$OPTARG;;
	S)	szTapeServer=$OPTARG;;
	j)	szSesamJob=$OPTARG;;
	T)	szTarget=$OPTARG;;
	R)	szCatalog=$OPTARG;;
	m)	szMediaPool=$OPTARG;;
	v)	szLogLevel=$OPTARG;;
        W)      szComTimeout=$OPTARG;;
	p)	iChannelMax=$OPTARG;;
	d)	szDrive=$OPTARG;;
  u)  szBlockSize=$OPTARG;;
        o|a)    if echo $OPTARG | grep -i "DBID=" >/dev/null
                then
                  szDBID=`echo $OPTARG | cut -f2 -d '='`
                  echo "got DBID $szDBID"
                fi 
                if echo $OPTARG | grep -i "until=" >/dev/null
                then
                  szDatetime=`echo $OPTARG | cut -f2 -d '='`
                  szUntil="set until time \"TO_DATE('$szDatetime', 'YYYYMMDD-HH24:MI:SS')\";"
                fi
                szOption="$szOption $OPTARG";;
	\?)	Usage
		exit 1;;
	esac
done
shift `expr $OPTIND - 1`
szTablespace=$*

szLogLevel=${szLogLevel:-"0"}
szCatalog=${szCatalog:-"nocatalog"}
[ $szCatalog = "nocatalog" ] || szCatalog="catalog $szCatalog"

#TBD check for empty arguments
#argument parsing done
#
if [[ $szOperation = "" ]]
then 
  Usage
  exit 1
fi

if [[ $szBlockSize = "" ]]
then
  szBlockSize = "262144"
fi

if [ $szCmdFile ]
then 
  my_echo "3500: Info:  Executing RMAN script: $szCmdFile"
  my_echo "3500: Info:  Executing 'rman $szCmdFile'"
  exec_rman_cmdfile $szCmdFile
  exit 0
fi

# Export sesam variables, if any
if [[ $szDrive != "" ]]
then
	export szDriveNum=$szDrive
fi

if [[ $szMediaPool != "" ]]
then
	export szMediaPool=$szMediaPool
fi

if [[ $szTapeServer != "" ]]
then
	export szTapeServer=$szTapeServer
fi

# If -C switch is not given then use -S as Sesam Server
# If -S switch is not given then use -C as Tape Server
if [[ $szSesamServer = "" ]]
then
	szSesamServer=$szTapeServer
else
  if [[ $szTapeServer = "" ]]
  then
	szTapeServer=$szSesamServer
  fi
fi

if [[  $szTablespace = "" ]]
then 
  szBackupSrc="database"
else
  szBackupSrc="tablespace \"$szTablespace\""
fi

szTarget=${szTarget:-$ORACLE_SID}
if `echo ${szTarget}|grep @ >/dev/null 2>&1`
then
  :
else
  export ORACLE_SID=$szTarget
  szTarget=""   # avoid connect target {ORACLE_SID}
fi

szSbtLog=$ORACLE_HOME/sob/sobsbt_${ORACLE_SID}_$$.log

if [[  $szDrive = "DISK" ]]
then
  szBckTarget="TYPE disk;"
  # Destination directory for DISK backup. / at the and is important
  szDiskDestination="$ORACLE_HOME/sob/backup/"
else
  szBckTarget=" type 'SBT' parms='BLKSIZE=$szBlockSize ENV=(SESAM_SERVER=$szSesamServer,SESAM_TAPE_SERVER=$szTapeServer,SESAM_JOB=$szSesamJob,SESAM_DRIVE=$szDriveNum,SESAM_POOL=$szMediaPool,SOB_TRACE=$szLogLevel,SOB_LOGFILE=$szSbtLog,SESAM_COM_TIMEOUT=$szComTimeout)';" 
fi


### -o delete >> Delete archivelogs when successfully backed up
if  echo $szOption | grep "delete" >/dev/null
then
  ### -o delete_later >> Delete Redologs when successfully backed up 3 times with SBT
  if  echo $szOption | grep "delete_later" >/dev/null
  then
    szDeleteLater="ALLOCATE CHANNEL FOR MAINTENANCE DEVICE TYPE DISK;
          DELETE NOPROMPT ARCHIVELOG ALL BACKED UP 3 TIMES TO DEVICE TYPE sbt;
          RELEASE CHANNEL;"
  else
    szDeleteInput="delete input"
  fi
fi

### -o offline
if  echo $szOption | grep "offline" >/dev/null
then
  echo "ATTENTION: Got -o offline so database will be shutdowned!"
  szShutdown="shutdown immediate"
  szStartup="startup mount"
  szStartupAfter="alter database open;"
fi

### -o controlfile
if  echo $szOption | grep  "controlfile" >/dev/null
then
  if [ "$szCatalog" = "nocatalog" ] 
  then
    if [ "$szDBID" = "" ]
    then
      echo "ERROR: Controlfile restore needs DBID if called with nocatalog"
      echo "Use option -o controlfile_DBID={DBID}"
      Usage
      exit 1
    fi
    szStartup="startup nomount;
       SET DBID $szDBID;"
    szRestoreCtrl="
       restore controlfile FROM AUTOBACKUP 
         MAXSEQ $szMaxSeq MAXDAYS $szMaxDays;
       restore spfile;
       alter database mount;"
  else
    szStartup="startup nomount"
    szRestoreCtrl="restore controlfile;
       restore spfile;
       alter database mount;"
  fi
fi

### -o recover
if  echo $szOption | grep "recover" >/dev/null
then
  szOptionRecoverOrValidate="recover $szBackupSrc;"
elif echo $szOption | grep "validate" >/dev/null
then
  szValidate="validate"
  szOptionRecoverOrValidate="restore archivelog all validate;"
fi


szLevel=${szLevel:-copy}

if [ `expr $szLevel : '.*arch.*'` -eq 0 -a  "$szDeleteInput" = "delete input" ]
then
   echo "Invalid usage"
   Usage
   exit 1 
fi


if [[ $szOperation = "Backup" ]]
then 
szLevel=${szLevel:-copy}
case $szLevel in 
    copy*) szBackupCmd="backup full"
           szFormatBackup="${szDiskDestination}Oc_%d_%u.dbf" 
           ;;
    full*) szBackupCmd="backup incremental level 0"
           szFormatBackup="${szDiskDestination}Of_%d_%u.dbf"
           ;;
    diff*) szBackupCmd="backup incremental level 1 cumulative"
           szFormatBackup="${szDiskDestination}Od%d%u" 
           ;;
    incr*) szBackupCmd="backup incremental level 1"
           szFormatBackup="${szDiskDestination}Oi%d%u" 
           ;;
    arch*) szBackupSrc="archivelog all $szDeleteInput"
           szBackupCmd="backup"
           szFormatBackup="${szDiskDestination}Oa%d%u"
           ;;
esac
case $szLevel in 
    copy*arch|full*arch|diff*arch|incr*arch)
# In ORACLE 9i not necessary:
#           szArch="sql 'alter system archive log current';
# backup format 'Oa_%d_%u.log' (archivelog all $szDeleteInput);"
           szArch="backup format 'Oa_%d_%u.log' (archivelog all $szDeleteInput);"
           ;;
esac
fi


if [[ $szOperation = "Restore" ]]
then 
  iChannelMax=1
  szBackupCmd="restore"
  szLevel="rest"
fi

# Allocate multiple channels

iChannelMax=${iChannelMax:-1}

count=1
while [ $count -le $iChannelMax ]
  do
    channels=$channels"ALLOCATE channel ${ORACLE_SID}$count $szBckTarget"
    re_channels=$re_channels"release channel ${ORACLE_SID}$count;"
    count=`expr $count + 1`
done


my_echo "3500: Info:  Oracle SID       : $szTarget"
my_echo "3500: Info:  export Oracle SID: $ORACLE_SID"
my_echo "3500: Info:  Backup source    : $szBackupSrc"
my_echo "3500: Info:  Sesam Server     : $szSesamServer"
my_echo "3500: Info:  Sesam Job        : $szSesamJob"
my_echo "3500: Info:  Sesam Mediapool  : $szMediaPool"
my_echo "3500: Info:  Backup level     : $szLevel ----> $szBackupCmd"
my_echo "3500: Info:  Backup channels  : $iChannelMax"

# Run Operation
iStatus=1
my_echo "3500: Info:  Building and running RMAN script $szLevel"


if [ "$szLogLevel" -gt 0 ]
then
   set -x
fi

case $szLevel in
        arch) 
	# 	
# In ORACLE 9i not necessary:
#        sql 'alter system archive log current';
        echo "	connect target $szTarget
	run { 
	`echo $channels` 
	`echo $szBackupCmd $szBackupSrc` 
	format '$szFormatBackup'; 
	`echo $re_channels` 
	}
        $szDeleteLater
	" | tee -a $szRmanLog | $szRmanCMD $szCatalog
	iStatus=$?
        ;;

        copy*|full*|diff*|incr*)
        #
        echo "connect target $szTarget
         CONFIGURE CONTROLFILE AUTOBACKUP ON;
         report schema;
        `echo $szShutdown`
        `echo $szStartup`
        run {
        `echo $channels`
        $szBackupCmd $szBackupSrc 
        format '$szFormatBackup';
        $szArch
        `echo $re_channels`
        }
        $szDeleteLater
        `echo $szStartupAfter`
	" | tee -a $szRmanLog | $szRmanCMD $szCatalog
	iStatus=$?
	;;
	rest*)
	echo "connect target $szTarget
        $szStartup
	run {
	$channels $szUntil $szRestoreCtrl
	$szBackupCmd $szBackupSrc $szValidate;
	$szOptionRecoverOrValidate
	$re_channels
	}
	" | tee -a $szRmanLog | $szRmanCMD $szCatalog
	iStatus=$?
	;;

	*)			
        Usage
esac

my_echo "3500: Removing old log files..."
find "$ORACLE_HOME/sob/" -type f -mtime +14 -a -name "*.log" -exec echo {} \; 
find "$ORACLE_HOME/sob/" -type f -mtime +14 -a -name "*.log" -exec rm {} \;

set +x   ## reset trace

if [[  $iStatus = "0" ]]
  then
    my_echo "3000: Info:  RMAN returned successfully. Status: $iStatus "
  else
    my_echo "1000: Error: RMAN returned with error. Status: $iStatus "
    my_echo ""
    my_echo "3500: ---------------- begin of sbt log ---------------- "
    cat $szSbtLog >&2
    my_echo "3500: ----------------  end of sbt log  ---------------- "
    my_echo "3500: "
    my_echo "1000: Error: RMAN returned with error. Status: $iStatus "
  fi


exit $iStatus

